"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const micromatch_1 = __importDefault(require("micromatch"));
const is_glob_1 = __importDefault(require("is-glob"));
const resolveImportPath_1 = __importDefault(require("./resolveImportPath"));
/**
 * pathのmatcher。
 * eslintrcで設定できる値は以下のケースを扱う
 * - globパターン指定
 * - globパターン以外の場合 => 前方部分一致
 */
const isMatch = (str, pattern) => (0, is_glob_1.default)(pattern) ? micromatch_1.default.isMatch(str, pattern) : str.startsWith(pattern);
exports.default = {
    meta: {
        type: 'suggestion',
        messages: {
            importNotAllowed: 'import {{importPath}} is not allowed from {{relativeFilePath}}.',
        },
        schema: [
            {
                type: 'array',
                items: [
                    {
                        type: 'object',
                        properties: {
                            module: {
                                type: 'string',
                            },
                            allowReferenceFrom: {
                                type: 'array',
                                items: [
                                    {
                                        type: 'string',
                                    },
                                ],
                            },
                            allowSameModule: {
                                type: 'boolean',
                            },
                            allowTypeImport: {
                                type: 'boolean',
                            },
                        },
                    },
                ],
            },
            {
                type: 'object',
                properties: {
                    resolveRelativeImport: {
                        type: 'boolean',
                    },
                },
            },
        ],
    },
    create: (context) => {
        const dependencies = context.options[0];
        // eslint-disable-next-line no-magic-numbers
        const options = (context.options.length > 1 ? context.options[1] : {});
        const resolveRelativeImport = options.resolveRelativeImport;
        const allowTypeImport = options.allowTypeImport;
        function checkImport(node) {
            if (allowTypeImport && node.importKind === 'type') {
                return;
            }
            const fileFullPath = context.getFilename();
            const relativeFilePath = path_1.default.relative(process.cwd(), fileFullPath);
            const importPath = (0, resolveImportPath_1.default)(node.source.value, resolveRelativeImport ? relativeFilePath : undefined);
            dependencies
                .filter((dependency) => isMatch(importPath, dependency.module))
                .forEach((dependency) => {
                const isAllowed = 
                // 参照元が許可されている
                dependency.allowReferenceFrom.some((allowPath) => isMatch(relativeFilePath, allowPath)) || // または同一モジュール間の参照が許可されている場合
                    (dependency.allowSameModule && isMatch(relativeFilePath, dependency.module)) ||
                    (dependency.allowTypeImport && node.importKind === 'type');
                if (!isAllowed) {
                    context.report({
                        node,
                        messageId: 'importNotAllowed',
                        data: { importPath, relativeFilePath },
                    });
                }
            });
        }
        return {
            ImportDeclaration: checkImport,
        };
    },
};
